@@ -48,7 +48,7 @@ db_dump_main(argc, argv)
 	extern char *optarg;
 	extern int optind, __db_getopt_reset;
 	DB_ENV	*dbenv;
-	DB *dbp;
+	DB *dbp, *dbvp;
 	db_pgno_t first, last;
 	u_int32_t cache;
 	int ch;
@@ -65,7 +65,7 @@ db_dump_main(argc, argv)
 		return (ret);
 
 	dbenv = NULL;
-	dbp = NULL;
+	dbp = dbvp = NULL;
 	exitval = lflag = mflag = nflag = pflag = rflag = Rflag = sflag = 0;
 	first = last = PGNO_INVALID;
 	keyflag = 0;
@@ -291,6 +291,10 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 		goto done;
 	}
 
+	if (db_create(&dbvp, dbenv, 0) != 0 ||
+		dbvp->verify(dbvp, filename, dbname, stdout, DB_NOORDERCHK) != 0)
+		goto err;
+
 	if ((ret = dbp->open(dbp, NULL,
 	    filename, dbname, DB_UNKNOWN, DB_RDWRMASTER|DB_RDONLY, 0)) != 0) {
 		dbp->err(dbp, ret, DB_STR_A("5115", "open: %s", "%s"),
