@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2000, 2015 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -10,7 +10,9 @@ package com.sleepycat.compat;
 
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.IOException;
 import java.util.Comparator;
+import java.util.regex.Pattern;
 
 import com.sleepycat.db.Cursor;
 import com.sleepycat.db.CursorConfig;
@@ -21,6 +23,7 @@ import com.sleepycat.db.DatabaseException;
 import com.sleepycat.db.DatabaseType;
 import com.sleepycat.db.Environment;
 import com.sleepycat.db.EnvironmentConfig;
+import com.sleepycat.db.ErrorHandler;
 import com.sleepycat.db.LockDetectMode;
 import com.sleepycat.db.LockMode;
 import com.sleepycat.db.OperationStatus;
@@ -511,6 +514,48 @@ public class DbCompat {
 
     public static boolean hasCaseInsensitiveOnDiskDbFile(){
         return !fSystemCaseSensitive;
-    } 
+    }
+
+    public static void enableDeadlockDetection(EnvironmentConfig envConfig,
+                                               boolean isCDB) {
+        if (isCDB) {
+            envConfig.setCDBLockAllDatabases(true);
+        } else {
+            envConfig.setLockDetectMode(LockDetectMode.MAXWRITE);
+        }
+    }
+
+    public static Object getErrorHandler(Environment env)
+        throws DatabaseException {
+        return env.getConfig().getErrorHandler();
+    }
+
+    public static void setErrorHandler(Environment env, Object errHandler)
+        throws DatabaseException {
+        EnvironmentConfig config = env.getConfig();
+        config.setErrorHandler((ErrorHandler) errHandler);
+        env.setConfig(config);
+    }
+
+    public static void suppressError(Environment env, final Pattern errPattern)
+        throws DatabaseException{
+        if (errPattern != null) {
+            final EnvironmentConfig config = env.getConfig();
+            ErrorHandler handler = new ErrorHandler() {
+                public void error(Environment environment, String errpfx, String msg) {
+                    if (!errPattern.matcher(msg).matches()) {
+                        try {
+                            config.getErrorStream().write(msg.getBytes());
+                            config.getErrorStream().write("\n".getBytes());
+                        } catch (IOException e) {
+                            e.printStackTrace();
+                        }
+                    }
+                }
+            };
+            config.setErrorHandler(handler);
+            env.setConfig(config);
+        }
+    }
 
 }
