@@ -70,6 +70,10 @@ abstract class BasicIndex<K, E> implements EntityIndex<K, E> {
         keyAdapter = new KeyValueAdapter(keyClass, keyBinding);
     }
 
+    public Database getDatabase() {
+        return db;
+    }
+
     /*
      * Of the EntityIndex methods only get()/map()/sortedMap() are not
      * implemented here and therefore must be implemented by subclasses.
@@ -121,6 +125,7 @@ abstract class BasicIndex<K, E> implements EntityIndex<K, E> {
         }
     }
 
+
     public boolean delete(K key)
         throws DatabaseException {
 
