@@ -585,6 +585,13 @@ import com.sleepycat.db.Transaction;
  */
 public interface EntityIndex<K, V> {
 
+    /**
+     * Returns the underlying database for this index.
+     *
+     * @return the database.
+     */
+    Database getDatabase();
+
     /**
      * Checks for existence of a key in this index.
      *
@@ -662,12 +669,6 @@ public interface EntityIndex<K, V> {
     /**
      * Returns a non-transactional count of the entities in this index.
      *
-     * <p>This operation is faster than obtaining a count by scanning the index
-     * manually, and will not perturb the current contents of the cache.
-     * However, the count is not guaranteed to be accurate if there are
-     * concurrent updates. Note that this method does scan a significant
-     * portion of the index and should be considered a fairly expensive
-     * operation.</p>
      *
      * @return the number of entities in this index.
      *
@@ -677,6 +678,7 @@ public interface EntityIndex<K, V> {
     long count()
         throws DatabaseException;
 
+
     /**
      * Deletes all entities with a given index key.
      *
