@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2002, 2015 Oracle and/or its affiliates.  All rights reserved.
  *
  */
 
@@ -16,6 +16,7 @@ import com.sleepycat.collections.StoredSortedMap;
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.db.Cursor;
 import com.sleepycat.db.CursorConfig;
+import com.sleepycat.db.Database;
 import com.sleepycat.db.DatabaseEntry;
 import com.sleepycat.db.DatabaseException;
 import com.sleepycat.db.Environment;
@@ -83,6 +84,10 @@ class SubIndex<PK, E> implements EntityIndex<PK, E> {
         entityAdapter = secIndex.entityAdapter;
     }
 
+    public Database getDatabase() {
+        return db;
+    }
+
     public boolean contains(PK key)
         throws DatabaseException {
 
@@ -141,6 +146,7 @@ class SubIndex<PK, E> implements EntityIndex<PK, E> {
         }
     }
 
+
     public boolean delete(PK key)
         throws DatabaseException {
 
