@@ -188,7 +188,7 @@ public class RangeCursor implements Cloneable {
     }
 
     /**
-     * If the operation succeded, leaves the duped cursor in place and closes
+     * If the operation succeeded, leaves the duped cursor in place and closes
      * the oldCursor.  If the operation failed, moves the oldCursor back in
      * place and closes the duped cursor.  oldCursor may be null if
      * beginOperation was not called, in cases where we don't need to dup
@@ -267,34 +267,35 @@ public class RangeCursor implements Cloneable {
             endOperation(null, status, null, null, null);
             return status;
         }
+        if (pkRange != null && pkRange.isSingleKey()) {
+            KeyRange.copy(range.beginKey, privKey);
+            KeyRange.copy(pkRange.beginKey, privPKey);
+            status = doGetSearchBoth(lockMode);
+            endOperation(null, status, key, pKey, data);
+            return status;
+        }
         if (pkRange != null) {
             KeyRange.copy(range.beginKey, privKey);
-            if (pkRange.singleKey) {
-                KeyRange.copy(pkRange.beginKey, privPKey);
-                status = doGetSearchBoth(lockMode);
-                endOperation(null, status, key, pKey, data);
-            } else {
-                status = OperationStatus.NOTFOUND;
-                Cursor oldCursor = beginOperation();
-                try {
-                    if (pkRange.beginKey == null || !sortedDups) {
-                        status = doGetSearchKey(lockMode);
-                    } else {
-                        KeyRange.copy(pkRange.beginKey, privPKey);
-                        status = doGetSearchBothRange(lockMode);
-                        if (status == OperationStatus.SUCCESS &&
-                            !pkRange.beginInclusive &&
-                            pkRange.compare(privPKey, pkRange.beginKey) == 0) {
-                            status = doGetNextDup(lockMode);
-                        }
-                    }
+            status = OperationStatus.NOTFOUND;
+            Cursor oldCursor = beginOperation();
+            try {
+                if (pkRange.beginKey == null || !sortedDups) {
+                    status = doGetSearchKey(lockMode);
+                } else {
+                    KeyRange.copy(pkRange.beginKey, privPKey);
+                    status = doGetSearchBothRange(lockMode);
                     if (status == OperationStatus.SUCCESS &&
-                        !pkRange.check(privPKey)) {
-                        status = OperationStatus.NOTFOUND;
+                        !pkRange.beginInclusive &&
+                        pkRange.compare(privPKey, pkRange.beginKey) == 0) {
+                        status = doGetNextDup(lockMode);
                     }
-                } finally {
-                    endOperation(oldCursor, status, key, pKey, data);
                 }
+                if (status == OperationStatus.SUCCESS &&
+                    !pkRange.check(privPKey)) {
+                    status = OperationStatus.NOTFOUND;
+                }
+            } finally {
+                endOperation(oldCursor, status, key, pKey, data);
             }
         } else if (range.singleKey) {
             KeyRange.copy(range.beginKey, privKey);
@@ -326,6 +327,15 @@ public class RangeCursor implements Cloneable {
         return status;
     }
 
+    /**
+     * This method will restart the operation when a key range is used and an
+     * insertion at the end of the key range is performed in another thread.
+     * The restarts are needed because a sequence of cursor movements is
+     * performed, and serializable isolation cannot be relied on to prevent
+     * insertions in other threads. Without the restarts, getLast could return
+     * NOTFOUND when keys in the range exist. This may only be an issue for JE
+     * since it uses record locking, while DB core uses page locking.
+     */
     public OperationStatus getLast(DatabaseEntry key,
                                    DatabaseEntry pKey,
                                    DatabaseEntry data,
@@ -333,79 +343,260 @@ public class RangeCursor implements Cloneable {
         throws DatabaseException {
 
         OperationStatus status = OperationStatus.NOTFOUND;
+
         if (!range.hasBound()) {
             setParams(key, pKey, data);
             status = doGetLast(lockMode);
             endOperation(null, status, null, null, null);
             return status;
         }
+
         Cursor oldCursor = beginOperation();
         try {
             if (pkRange != null) {
-                KeyRange.copy(range.beginKey, privKey);
-                boolean doLast = false;
-                if (!sortedDups) {
-                    status = doGetSearchKey(lockMode);
-                } else if (pkRange.endKey == null) {
-                    doLast = true;
-                } else {
-                    KeyRange.copy(pkRange.endKey, privPKey);
-                    status = doGetSearchBothRange(lockMode);
-                    if (status == OperationStatus.SUCCESS) {
-                        if (!pkRange.endInclusive ||
-                            pkRange.compare(pkRange.endKey, privPKey) != 0) {
-                            status = doGetPrevDup(lockMode);
-                        }
-                    } else {
-                        KeyRange.copy(range.beginKey, privKey);
-                        doLast = true;
-                    }
-                }
-                if (doLast) {
-                    status = doGetSearchKey(lockMode);
-                    if (status == OperationStatus.SUCCESS) {
-                        status = doGetNextNoDup(lockMode);
-                        if (status == OperationStatus.SUCCESS) {
-                            status = doGetPrev(lockMode);
-                        } else {
-                            status = doGetLast(lockMode);
-                        }
-                    }
-                }
+                status = getLastInPKeyRange(lockMode);
+
+                /* Final check on candidate key and pKey value. */
                 if (status == OperationStatus.SUCCESS &&
-                    !pkRange.check(privPKey)) {
+                    !(range.check(privKey) && pkRange.check(privPKey))) {
                     status = OperationStatus.NOTFOUND;
                 }
-            } else if (range.endKey == null) {
-                status = doGetLast(lockMode);
             } else {
-                KeyRange.copy(range.endKey, privKey);
-                status = doGetSearchKeyRange(lockMode);
-                if (status == OperationStatus.SUCCESS) {
-                    if (range.endInclusive &&
-                        range.compare(range.endKey, privKey) == 0) {
-                        /* Skip this step if dups are not configured? */
-                        status = doGetNextNoDup(lockMode);
-                        if (status == OperationStatus.SUCCESS) {
-                            status = doGetPrev(lockMode);
-                        } else {
-                            status = doGetLast(lockMode);
-                        }
-                    } else {
-                        status = doGetPrev(lockMode);
-                    }
-                } else {
-                    status = doGetLast(lockMode);
+                status = getLastInKeyRange(lockMode);
+
+                /* Final check on candidate key value. */
+                if (status == OperationStatus.SUCCESS &&
+                    !range.check(privKey)) {
+                    status = OperationStatus.NOTFOUND;
                 }
             }
-            if (status == OperationStatus.SUCCESS &&
-                !range.checkBegin(privKey, true)) {
-                status = OperationStatus.NOTFOUND;
-            }
+
+            return status;
         } finally {
             endOperation(oldCursor, status, key, pKey, data);
         }
-        return status;
+    }
+
+    /**
+     * Performs getLast operation when a main key range is specified but
+     * pkRange is null. Does everything but the final checks for key in range,
+     * i.e., when SUCCESS is returned the caller should do the final check.
+     */
+    private OperationStatus getLastInKeyRange(LockMode lockMode)
+        throws DatabaseException {
+
+        /* Without an endKey, getLast returns the candidate record. */
+        if (range.endKey == null) {
+            return doGetLast(lockMode);
+        }
+
+        /*
+         * K stands for the main key at the cursor position in the comments
+         * below.
+         */
+        while (true) {
+            KeyRange.copy(range.endKey, privKey);
+            OperationStatus status = status = doGetSearchKeyRange(lockMode);
+
+            if (status == OperationStatus.SUCCESS) {
+
+                /* Found K >= endKey. */
+                if (range.endInclusive &&
+                    range.compare(range.endKey, privKey) == 0) {
+
+                    /* K == endKey and endKey is inclusive. */
+
+                    if (!sortedDups) {
+                        /* If dups are not configured, we're done. */
+                        return OperationStatus.SUCCESS;
+                    }
+
+                    /*
+                     * If there are dups, we're positioned at endKey's first
+                     * dup and we want to move to its last dup. Move to the
+                     * first dup for the next main key (getNextNoDup) and then
+                     * the prev record. In the absence of insertions by other
+                     * threads, the prev record is the last dup for endKey.
+                     */
+                    status = doGetNextNoDup(lockMode);
+                    if (status == OperationStatus.SUCCESS) {
+
+                        /*
+                         * K > endKey. Move backward to the last dup for
+                         * endKey.
+                         */
+                        status = doGetPrev(lockMode);
+                    } else {
+
+                        /*
+                         * endKey is the last main key in the DB. Its last dup
+                         * is the last key in the DB.
+                         */
+                        status = doGetLast(lockMode);
+                    }
+                } else {
+
+                    /*
+                     * K > endKey or endKey is exclusive (and K >= endKey). In
+                     * both cases, moving to the prev key finds the last key in
+                     * the range, whether or not there are dups.
+                     */
+                    status = doGetPrev(lockMode);
+                }
+            } else {
+
+                /*
+                 * There are no keys >= endKey in the DB. The last key in the
+                 * range is the last key in the DB.
+                 */
+                status = doGetLast(lockMode);
+            }
+
+            if (status != OperationStatus.SUCCESS) {
+                return status;
+            }
+
+            if (!range.checkEnd(privKey, true)) {
+
+                /*
+                 * The last call above (getPrev or getLast) returned a key
+                 * outside the endKey range. Another thread must have inserted
+                 * this key. Start over.
+                 */
+                continue;
+            }
+
+            return status;
+        }
+    }
+
+    /**
+     * Performs getLast operation when both a main key range (which must be a
+     * single key range) and a pkRange are specified. Does everything but the
+     * final checks for key and pKey in range, i.e., when SUCCESS is returned
+     * the caller should do the final two checks.
+     */
+    private OperationStatus getLastInPKeyRange(LockMode lockMode)
+        throws DatabaseException {
+
+        /* We can do an exact search when range and pkRange are single keys. */
+        if (pkRange.isSingleKey()) {
+            KeyRange.copy(range.beginKey, privKey);
+            KeyRange.copy(pkRange.beginKey, privPKey);
+            return doGetSearchBoth(lockMode);
+        }
+
+        /*
+         * When dups are not configured, getSearchKey for the main key returns
+         * the only possible candidate record.
+         */
+        if (!sortedDups) {
+            KeyRange.copy(range.beginKey, privKey);
+            return doGetSearchKey(lockMode);
+        }
+
+        /*
+         * K stands for the main key and D for the duplicate (data item) at the
+         * cursor position in the comments below
+         */
+        while (true) {
+
+            if (pkRange.endKey != null) {
+
+                KeyRange.copy(range.beginKey, privKey);
+                KeyRange.copy(pkRange.endKey, privPKey);
+                OperationStatus status = doGetSearchBothRange(lockMode);
+
+                if (status == OperationStatus.SUCCESS) {
+
+                    /* Found D >= endKey. */
+                    if (!pkRange.endInclusive ||
+                        pkRange.compare(pkRange.endKey, privPKey) != 0) {
+
+                        /*
+                         * D > endKey or endKey is exclusive (and D >= endKey).
+                         * In both cases, moving to the prev dup finds the last
+                         * key in the range.
+                         */
+                        status = doGetPrevDup(lockMode);
+
+                        if (status != OperationStatus.SUCCESS) {
+                            return status;
+                        }
+
+                        if (!pkRange.checkEnd(privPKey, true)) {
+
+                            /*
+                             * getPrevDup returned a key outside the endKey
+                             * range. Another thread must have inserted this
+                             * key. Start over.
+                             */
+                            continue;
+                        }
+                    }
+                    /* Else D == endKey and endKey is inclusive. */
+
+                    return OperationStatus.SUCCESS;
+                }
+                /* Else there are no dups >= endKey.  Fall through. */
+            }
+
+            /*
+             * We're here for one of two reasons:
+             *  1. pkRange.endKey == null.
+             *  2. There are no dups >= endKey for the main key (status
+             *     returned by getSearchBothRange above was not SUCCESS).
+             * In both cases, the last dup in the range is the last dup for the
+             * main key.
+             */
+            KeyRange.copy(range.beginKey, privKey);
+            OperationStatus status = doGetSearchKey(lockMode);
+
+            if (status != OperationStatus.SUCCESS) {
+                return status;
+            }
+
+            /*
+             * K == the main key and D is its first dup. We want to move to its
+             * last dup. Move to the first dup for the next main key;
+             * (getNextNoDup) and then the prev record. In the absence of
+             * insertions by other threads, the prev record is the last dup for
+             * the main key.
+             */
+            status = doGetNextNoDup(lockMode);
+
+            if (status == OperationStatus.SUCCESS) {
+
+                /*
+                 * K > main key and D is its first dup. Move to the prev record
+                 * which should be the last dup for the main key.
+                 */
+                status = doGetPrev(lockMode);
+            } else {
+
+                /*
+                 * The main key specified is the last main key in the DB. Its
+                 * last dup is the last record in the DB.
+                 */
+                status = doGetLast(lockMode);
+            }
+
+            if (status != OperationStatus.SUCCESS) {
+                return status;
+            }
+
+            if (!range.checkEnd(privKey, true)) {
+
+                /*
+                 * The last call above (getPrev or getLast) returned a key
+                 * outside the endKey range. Another thread must have inserted
+                 * this key. Start over.
+                 */
+                continue;
+            }
+
+            return OperationStatus.SUCCESS;
+        }
     }
 
     public OperationStatus getNext(DatabaseEntry key,
