@@ -15,6 +15,8 @@
 #include "dbinc/db_page.h"
 #include "dbinc/crypto.h"
 
+static void randomize __P((ENV *, void *, size_t));
+
 /*
  * __crypto_region_init --
  *	Initialize crypto.
@@ -110,7 +112,7 @@ __crypto_region_init(env)
 	 * existing one, we are done with the passwd in the env.  We smash
 	 * N-1 bytes so that we don't overwrite the nul.
 	 */
-	memset(dbenv->passwd, 0xff, dbenv->passwd_len-1);
+	randomize(env, dbenv->passwd, dbenv->passwd_len - 1);
 	__os_free(env, dbenv->passwd);
 	dbenv->passwd = NULL;
 	dbenv->passwd_len = 0;
@@ -135,9 +137,10 @@ __crypto_env_close(env)
 	dbenv = env->dbenv;
 
 	if (dbenv->passwd != NULL) {
-		memset(dbenv->passwd, 0xff, dbenv->passwd_len-1);
+		randomize(env, dbenv->passwd, dbenv->passwd_len - 1);
 		__os_free(env, dbenv->passwd);
 		dbenv->passwd = NULL;
+		dbenv->passwd_len = 0;
 	}
 
 	if (!CRYPTO_ON(env))
@@ -417,3 +420,45 @@ __crypto_set_passwd(env_src, env_dest)
 	sh_passwd = R_ADDR(infop, cipher->passwd);
 	return (__env_set_encrypt(env_dest->dbenv, sh_passwd, DB_ENCRYPT_AES));
 }
+
+/*
+ * randomize
+ *	
+ */
+static void
+randomize(env, base, size)
+	ENV *env;
+	void *base;
+	size_t size;
+{
+	size_t i, copysize;
+	u_int8_t  last, *p;
+	u_int32_t value;
+
+	last = ((u_int8_t *)base)[size];
+	for (i = 0, p = base; i < size; i += copysize, p += copysize) {
+		value = __os_random();
+		if ((copysize = (size - i)) > sizeof(int32_t))
+			copysize = sizeof(int32_t);
+		switch (copysize)
+		{
+		default:
+			memmove(p, &value, sizeof(int32_t));
+		    	break;
+		case 3:
+			p[2] = (u_int8_t)(value >> 16);
+			/* FALLTHROUGH */
+		case 2:
+			p[1] = (u_int8_t)(value >> 8);
+			/* FALLTHROUGH */
+		case 1:
+			p[0] = (u_int8_t)(value);
+			break;
+		case 0:
+			DB_ASSERT(env, "randomize size 0?");
+			break;
+		}
+
+	}
+	DB_ASSERT(env, last == *p);
+}
