@@ -486,15 +486,17 @@ typedef volatile u_int32_t tsl_t;
  * ARM/gcc assembly.
  *********************************************************************/
 #ifdef HAVE_MUTEX_ARM_GCC_ASSEMBLY
-typedef unsigned char tsl_t;
+typedef unsigned int tsl_t;
 
 #ifdef LOAD_ACTUAL_MUTEX_CODE
 /* gcc/arm: 0 is clear, 1 is set. */
 #define	MUTEX_SET(tsl) ({						\
-	int __r;							\
+	register tsl_t __r;						\
 	__asm__ volatile(						\
-		"swpb	%0, %1, [%2]\n\t"				\
-		"eor	%0, %0, #1\n\t"					\
+		"ldrex		%0, [%2]\n\t"				\
+		"cmp		%0, %1\n\t"				\
+		"strexne	%0, %1, [%2]\n\t"			\
+		"eor		%0, %0, #1\n\t"				\
 	    : "=&r" (__r)						\
 	    : "r" (1), "r" (tsl)					\
 	    );								\
@@ -503,6 +505,44 @@ typedef unsigned char tsl_t;
 
 #define	MUTEX_UNSET(tsl)	(*(volatile tsl_t *)(tsl) = 0)
 #define	MUTEX_INIT(tsl)         (MUTEX_UNSET(tsl), 0)
+#define	MUTEX_MEMBAR(x)	\
+	({ __asm__ volatile ("dsb"); })
+#define	MEMBAR_ENTER() \
+	({ __asm__ volatile ("dsb"); })
+#define	MEMBAR_EXIT() \
+	({ __asm__ volatile ("dsb"); })
+#endif
+#endif
+
+/*********************************************************************
+ * ARM64/gcc assembly.
+ *********************************************************************/
+#ifdef HAVE_MUTEX_ARM64_GCC_ASSEMBLY
+typedef unsigned int tsl_t;
+
+#ifdef LOAD_ACTUAL_MUTEX_CODE
+/* gcc/arm: 0 is clear, 1 is set. */
+#define	MUTEX_SET(tsl) ({						\
+	register tsl_t __r, __old;					\
+	__asm__ volatile(						\
+		"ldxr	%w1, [%3]\n\t"					\
+		"stxr	%w0, %w2, [%3]\n\t"				\
+		"orr	%w0, %w0, %w1\n\t"				\
+		"neg	%w0, %w0\n\t"					\
+	    : "=&r" (__r), "=r" (__old)					\
+	    : "r" (1), "r" (tsl)					\
+	    );								\
+	__r & 1;							\
+})
+
+#define	MUTEX_UNSET(tsl)	(*(volatile tsl_t *)(tsl) = 0)
+#define	MUTEX_INIT(tsl)         (MUTEX_UNSET(tsl), 0)
+#define	MUTEX_MEMBAR(x)	\
+	({ __asm__ volatile ("dsb sy"); })
+#define	MEMBAR_ENTER() \
+	({ __asm__ volatile ("dsb sy"); })
+#define	MEMBAR_EXIT() \
+	({ __asm__ volatile ("dsb sy"); })
 #endif
 #endif
 
