@@ -33,6 +33,12 @@ static void __env_clear_state __P((ENV *));
 	if (((ret) = (t_ret)) != 0) goto err
 #endif
 
+/*
+ * Set the default number of buckets to be 1/8th the number of
+ * thread control blocks.  This is rather arbitrary.
+ */
+#define DB_THREAD_INFOS_PER_BUCKET	8
+
 /*
  * __env_failchk_pp --
  *	ENV->failchk pre/post processing.
@@ -142,7 +148,8 @@ err:
 
 /*
  * __env_thread_size --
- *	Initial amount of memory for thread info blocks.
+ *	Calculate the initial amount of memory for thread info blocks.
+ *
  * PUBLIC: size_t __env_thread_size __P((ENV *, size_t));
  */
 size_t
@@ -155,44 +162,36 @@ __env_thread_size(env, other_alloc)
 	u_int32_t max;
 
 	dbenv = env->dbenv;
-	size = 0;
-
-	max = dbenv->thr_max;
-	if (dbenv->thr_init != 0) {
-		size =
-		    dbenv->thr_init * __env_alloc_size(sizeof(DB_THREAD_INFO));
-		if (max < dbenv->thr_init)
-			max = dbenv->thr_init;
-	} else if (max == 0 && ALIVE_ON(env)) {
-		if ((max = dbenv->tx_init) == 0) {
-			/*
-			 * They want thread tracking, but don't say how much.
-			 * Arbitrarily assume 1/10 of the remaining memory
-			 * or at least 100.  We just use this to size
-			 * the hash table.
-			 */
-			if (dbenv->memory_max != 0)
-				max = (u_int32_t)
-				    (((dbenv->memory_max - other_alloc) / 10) /
-					sizeof(DB_THREAD_INFO));
-			if (max < 100)
-				max = 100;
-		}
+	if ((max = dbenv->thr_max) < dbenv->thr_init)
+		max = dbenv->thr_init;
+	else if (max == 0 && ALIVE_ON(env) && (max = dbenv->tx_init) == 0) {
+		/*
+		 * They want thread tracking, but don't say how much.
+		 * Arbitrarily assume 1/10 of the remaining memory or at
+		 * least 100.
+		 */
+		if (dbenv->memory_max != 0)
+			max = (u_int32_t)
+			    (((dbenv->memory_max - other_alloc) / 10) /
+				sizeof(DB_THREAD_INFO));
+		if (max < 100)
+			max = 100;
 	}
-	/*
-	 * Set the number of buckets to be 1/8th the number of
-	 * thread control blocks.  This is rather arbitrary.
-	 */
 	dbenv->thr_max = max;
-	if (max != 0)
-		size += __env_alloc_size(sizeof(DB_HASHTAB) *
-		    __db_tablesize(max / 8));
+	env->thr_nbucket = __db_tablesize(max / DB_THREAD_INFOS_PER_BUCKET);
+	/*
+	 * Include space for thread control block hash table and enough
+	 * entries in each hash bucket for the initial number of threads.
+	 */
+	size = __env_alloc_size(env->thr_nbucket * sizeof(DB_HASHTAB));
+	size += dbenv->thr_init * __env_alloc_size(sizeof(DB_THREAD_INFO));
 	return (size);
 }
 
 /*
  * __env_thread_max --
- *	Return the amount of extra memory to hold thread information.
+ *	Return how much additional memory to reserve for threads.
+ *
  * PUBLIC: size_t __env_thread_max __P((ENV *));
  */
 size_t
@@ -200,24 +199,20 @@ __env_thread_max(env)
 	ENV *env;
 {
 	DB_ENV *dbenv;
-	size_t size;
+	size_t count;
 
 	dbenv = env->dbenv;
 
-	/*
-	 * Allocate space for thread info blocks.  Max is only advisory,
-	 * so we allocate 25% more.
+	/* 
+	 * Allow for the worst case number of configured thread control blocks,
+	 * plus 25%; then subtract the number of threads already allowed for.
 	 */
-	if (dbenv->thr_max > dbenv->thr_init) {
-		size = dbenv->thr_max - dbenv->thr_init;
-		size += size / 4;
-	} else {
-		dbenv->thr_max = dbenv->thr_init;
-		size = dbenv->thr_init / 4;
-	}
-
-	size = size * __env_alloc_size(sizeof(DB_THREAD_INFO));
-	return (size);
+	count = env->thr_nbucket * dbenv->thr_max;
+	if (count < dbenv->thr_init)
+		count = dbenv->thr_init;
+	count += count / 4;
+	return ((count - dbenv->thr_init) *
+	    __env_alloc_size(sizeof(DB_THREAD_INFO)));
 }
 
 /*
@@ -267,7 +262,8 @@ __env_thread_init(env, during_creation)
 		}
 		memset(thread, 0, sizeof(*thread));
 		renv->thread_off = R_OFFSET(infop, thread);
-		thread->thr_nbucket = __db_tablesize(dbenv->thr_max / 8);
+		thread->thr_nbucket =
+		    __db_tablesize(dbenv->thr_max / DB_THREAD_INFOS_PER_BUCKET);
 		if ((ret = __env_alloc(infop,
 		     thread->thr_nbucket * sizeof(DB_HASHTAB), &htab)) != 0)
 			return (ret);
@@ -455,6 +451,11 @@ struct __db_threadid {
 };
 
 /*
+ * __env_set_state --
+ *	Set the state of the current thread's entry in the thread information
+ *	table, alocating an entry if necessary, or look for its location without
+ *	modifying the table. Do nothing if locking has been turned off.
+ *
  * PUBLIC: int __env_set_state __P((ENV *, DB_THREAD_INFO **, DB_THREAD_STATE));
  */
 int
@@ -513,28 +514,34 @@ __env_set_state(env, ipp, state)
 #endif
 	}
 
-	/* A failchk thread must not block on a lock -- that would be faulty. */
-	if (state == THREAD_BLOCKED && ip != NULL)
-		DB_ASSERT(env, ip->dbth_state != THREAD_FAILCHK);
 	/*
-	 * If ipp is not null,  return the thread control block if found.
-	 * Check to ensure the thread of control has been registered.
+	 * THREAD_VERIFY does not add the pid+thread to the table. The caller
+	 * expects to find itself in the table already. If an ipp was passed in
+	 * store the ip location there. Return whether or not it was found.
+	 * In diagnostic mode a missing entry triggers the assert.
 	 */
 	if (state == THREAD_VERIFY) {
 		DB_ASSERT(env, ip != NULL && ip->dbth_state != THREAD_OUT);
-		if (ipp != NULL) {
-			if (ip == NULL) /* The control block wasn't found */
-				return (EINVAL);
+		if (ipp != NULL)
 			*ipp = ip;
-		}
-		return (0);
+		if (ip == NULL || ip->dbth_state == THREAD_OUT)
+			return (USR_ERR(env, EINVAL));
+		else
+			return (0);
 	}
 
 	*ipp = NULL;
 	ret = 0;
-	if (ip != NULL)
+	if (ip != NULL) {
+		/*
+		 * __env_set_state() gets called in many places, but a thread
+		 * doing failchk is only supposed to be called to set the state
+		 * to THREAD_OUT. Complain for any other state change.
+		 */
+		if (state != THREAD_OUT)
+			DB_ASSERT(env, ip->dbth_state != THREAD_FAILCHK);
 		ip->dbth_state = state;
-	else {
+	} else {
 		infop = env->reginfo;
 		renv = infop->primary;
 		thread = R_ADDR(infop, renv->thread_off);
@@ -542,9 +549,10 @@ __env_set_state(env, ipp, state)
 
 		/*
 		 * If we are passed the specified max, try to reclaim one from
-		 * our queue.  If failcheck has marked the slot not in use, we
+		 * our bucket.  If failcheck has marked the slot not in use, we
 		 * can take it, otherwise we must call is_alive before freeing
-		 * it.
+		 * it. We do not reclaim any entries from other buckets -- doing
+		 * so safely would require lookups to lock mtx_regenv.
 		 */
 		if (thread->thr_count >= thread->thr_max) {
 			SH_TAILQ_FOREACH(
@@ -566,10 +574,10 @@ __env_set_state(env, ipp, state)
 		     sizeof(DB_THREAD_INFO), &ip)) == 0) {
 			memset(ip, 0, sizeof(*ip));
 			/*
-			 * This assumes we can link atomically since we do
-			 * no locking here.  We never use the backpointer
-			 * so we only need to be able to write an offset
-			 * atomically.
+			 * As long as inserting at the head can be 'committed'
+			 * with a single memory write, this entry can be added
+			 * without requiring any concurrent searches to
+			 * do any locking.
 			 */
 			SH_TAILQ_INSERT_HEAD(
 			    &htab[indx], ip, dbth_links, __db_thread_info);
