@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1999, 2015 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -31,13 +31,15 @@ DB_GLOBALS __db_global_values = {
 
 	"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=", /* db_line */
 	{ 0 },				/* error_buf */
-	0,				/* uid_init */
-	0,				/* rand_next */
+	0,				/* random_seeded */
+#if defined(HAVE_RANDOM_R)
+	{ 0 },				/* random_r random_data */
+	{ 0 },				/* random_r state */
+#elif !defined(HAVE_RAND) && !defined(HAVE_RANDOM)
+	0,				/* rand/srand value */
+#endif
 	0,				/* fid_serial */
 	0,				/* db_errno */
-	0,				/* num_active_pids */
-	0,				/* size_active_pids */
-	NULL,                           /* active_pids */
 	NULL,                           /* saved_errstr */
 	"%m/%d %H:%M:%S",		/* strftime format for dates */
 #if defined(HAVE_ERROR_HISTORY)
