@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -137,12 +137,18 @@ __env_open(dbenv, db_home, flags, mode)
 	ENV *env;
 	u_int32_t orig_flags, retry_flags;
 	int recovery_failed, register_recovery, ret, t_ret;
+	char *old_passwd;
+	size_t old_passwd_len;
+	u_int32_t old_encrypt_flags;
 
 	ip = NULL;
 	env = dbenv->env;
 	recovery_failed = 1;
 	register_recovery = 0;
 	retry_flags = 0;
+	old_passwd = NULL;
+	old_passwd_len = 0;
+	old_encrypt_flags = 0;
 
 	/* Initial configuration. */
 	if ((ret = __env_config(dbenv, db_home, &flags, mode)) != 0)
@@ -179,6 +185,17 @@ __env_open(dbenv, db_home, flags, mode)
 			dbenv->is_alive = __envreg_isalive;
 		}
 
+		/* 
+		 * Backup the current key, because it would be consumed by
+		 * __envreg_register below
+		 */
+		if (dbenv->passwd != NULL) {
+			if ((ret = __os_strdup(env, dbenv->passwd, &old_passwd)) != 0)
+				goto err;
+			old_passwd_len = dbenv->passwd_len;
+			(void)__env_get_encrypt_flags(dbenv, &old_encrypt_flags);
+		}
+
 		F_SET(dbenv, DB_ENV_NOPANIC);
 		ret = __envreg_register(env, &register_recovery, flags);
 		dbenv->flags = orig_flags;
@@ -217,6 +234,15 @@ retry:	if (LF_ISSET(DB_RECOVER | DB_RECOVER_FATAL))
 		    orig_flags | retry_flags, 0)) != 0)
 			goto err;
 
+	/* Restore the database key. */
+	if (LF_ISSET(DB_REGISTER) && old_passwd != NULL) {
+		ret = __env_set_encrypt(dbenv, old_passwd, old_encrypt_flags);
+		memset(old_passwd, 0xff, old_passwd_len - 1);
+		__os_free(env, old_passwd);
+		if (ret != 0)
+			goto err;
+	}
+
 	DB_ASSERT(env, ret == 0);
 	if ((ret = __env_attach_regions(dbenv,
 	    flags, orig_flags | retry_flags, 1)) != 0)
@@ -235,13 +261,13 @@ retry:	if (LF_ISSET(DB_RECOVER | DB_RECOVER_FATAL))
 		 */
 		FAILCHK_THREAD(env, ip);
 		ret = __env_failchk_int(dbenv);
-		ENV_LEAVE(env, ip);
 		if (ret != 0) {
 			__db_err(env, ret,
 			    DB_STR("1595",
 			    "failchk crash after clean registry"));
 			goto err;
 		}
+		ENV_LEAVE(env, ip);
 	}
 
 err:	if (ret != 0)
@@ -564,9 +590,10 @@ __env_close_pp(dbenv, flags)
 
 		/* Close all underlying file handles. */
 		(void)__file_handle_cleanup(env);
+		ENV_LEAVE(env, ip);
+
 		dbenv->flags = flags_orig;
 		(void)__env_region_cleanup(env);
-		ENV_LEAVE(env, ip);
 
 		return (__env_panic_msg(env));
 	}
@@ -655,8 +682,11 @@ __env_close(dbenv, flags)
 			t_ret = dbp->alt_close(dbp, close_flags);
 		else
 			t_ret = __db_close(dbp, NULL, close_flags);
-		if (t_ret != 0 && ret == 0)
-			ret = t_ret;
+		if (t_ret != 0) {
+			if (ret == 0)
+				ret = t_ret;
+			break;
+		}
 	}
 
 	/*
@@ -961,7 +991,8 @@ __file_handle_cleanup(env)
 	while ((fhp = TAILQ_FIRST(&env->fdlist)) != NULL) {
 		__db_errx(env,
 		    DB_STR_A("1582", "Open file handle: %s", "%s"), fhp->name);
-		(void)__os_closehandle(env, fhp);
+		if (__os_closehandle(env, fhp) != 0)
+			break;
 	}
 	if (env->lockfhp != NULL)
 		env->lockfhp = NULL;
