@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996, 2014 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 1996, 2015 Oracle and/or its affiliates.  All rights reserved.
  *
  * $Id$
  */
@@ -583,8 +583,11 @@ __memp_env_refresh(env)
 not_priv:
 	/* Discard DB_MPOOLFILEs. */
 	while ((dbmfp = TAILQ_FIRST(&dbmp->dbmfq)) != NULL)
-		if ((t_ret = __memp_fclose(dbmfp, DB_FLUSH)) != 0 && ret == 0)
-			ret = t_ret;
+		if ((t_ret = __memp_fclose(dbmfp, DB_FLUSH)) != 0) {
+			if (ret == 0)
+				ret = t_ret;
+			break;
+		}
 
 	/* Discard DB_MPREGs. */
 	if (dbmp->pg_inout != NULL)
@@ -675,8 +678,11 @@ __memp_region_bhfree(infop)
 				if ((t_ret = __memp_bhfree(dbmp, infop,
 				    R_ADDR(dbmp->reginfo, bhp->mf_offset),
 				    hp, bhp, BH_FREE_FREEMEM |
-				    BH_FREE_UNLOCKED)) != 0 && ret == 0)
-					ret = t_ret;
+				    BH_FREE_UNLOCKED)) != 0) {
+				    	if (ret == 0)
+						ret = t_ret;
+					break;
+				}
 			}
 	}
 	MPOOL_REGION_LOCK(env, infop);
